% ------------------------------------------------------------------------- %
% lazy.Rd                                Lazy learning for local regression %
% ------------------------------------------------------------------------- %
                                                                             
% ========================================================================= %
% Lazy learning for local regression                                        %
% ------------------------------------------------------------------------- %
% Copyright (C) 1999, 2003 Mauro Birattari and Gianluca Bontempi            %
% ========================================================================= %
% This program is free software; you can redistribute it and/or modify it   %
% under the terms of the GNU General Public License as published by the     %
% Free Software Foundation; either version 2 of the License, or (at your    %
% option) any later version.                                                %
%                                                                           %
% This program is distributed in the hope that it will be useful, but       %
% WITHOUT ANY WARRANTY; without even the implied warranty of                %
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         %
% General Public License for more details.                                  %
%                                                                           %
% You should have received a copy of the GNU General Public License along   %
% with this program; if not, write to the Free Software Foundation, Inc.,   %
% 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.                  %
% ========================================================================= %

% ========================================================================= %
%             Mauro Birattari                     Gianluca Bontempi         %
%                IRIDIA                     Departement d'Informatique      %
%    Universite' Libre de Bruxelles       Universite' Libre de Bruxelles    %
%             mbiro@ulb.ac.be                     gbonte@ulb.ac.be          %
% ========================================================================= %

% $Id: lazy.Rd,v 1.11 2003/11/18 15:36:29 mbiro Exp $ %

\name{lazy}
\alias{lazy}
\alias{print.lazy}
\alias{print.summary.lazy}
\alias{summary.lazy}

\title{Lazy learning for local regression}

\description{By combining constant, linear, and quadratic local models,
  \code{lazy} estimates the value of an unknown multivariate function on
  the basis of a set of possibly noisy samples of the function itself.
  This implementation of lazy learning automatically adjusts the
  bandwidth on a query-by-query basis through a leave-one-out
  cross-validation.}

\usage{
lazy(formula, data=NULL, weights, subset, na.action,
        control=lazy.control(\dots), \dots)
}

\arguments{
  \item{formula}{A formula specifying the response and some numeric
          predictors.}

  \item{data}{An optional data frame within which to look first for the
          response, predictors, and weights (the latter will be
          ignored).}

  \item{weights}{Optional weights for each case (ignored).}

  \item{subset}{An optional specification of a subset of the data to be
          used.}

  \item{na.action}{The action to be taken with missing values in the response
          or predictors.  The default is to stop.}

  \item{control}{Control parameters: see \code{\link{lazy.control}}}.

  \item{\dots}{Control parameters can also be supplied directly.}
}

\details{For one or more query points, \code{lazy} estimates the value of
  an unknown multivariate function on the basis of a set of possibly
  noisy samples of the function itself.  Each sample is an input/output
  pair where the input is a vector and the output is a number.  For each
  query point, the estimation of the function is obtained by combining
  different local models.  Local models considered for combination by
  \code{lazy} are polynomials of zeroth, first, and second degree that
  fit a set of samples in the neighborhood of the query point. The
  neighbors are selected according to either the Manhattan or the
  Euclidean distance. It is possible to assign weights to the different
  directions of the input domain for modifying their importance in the
  computation of the distance.  The number of neighbors used for
  identifying local models is automatically adjusted on a query-by-query
  basis through a leave-one-out validations of models, each fitting a
  different numbers of neighbors.  The local models are identified using
  the recursive least-squares algorithm, and the leave-one-out
  cross-validation is obtained through the PRESS statistic.

  As the name \code{lazy} suggests, this function does not do
  anything\ldots apart from checking the options and properly packing
  the data. All the actual computation is done when a prediction is
  request for a specific query point, or for a set of query points: see
  \code{\link{predict.lazy}}.
}

\value{An object of class \code{lazy}.}

\references{
   D.W. Aha (1997) Editorial. \emph{Artificial Intelligence Review},
   \bold{11}(1--5), pp. 1--6. Special Issue on Lazy Learning.

   C.G. Atkeson, A.W. Moore, and S. Schaal (1997) Locally Weighted
   Learning. \emph{Artificial Intelligence Review}, \bold{11}(1--5),
   pp. 11--73. Special Issue on Lazy Learning.

   W.S. Cleveland, S.J. Devlin, and S.J. Grosse (1988) Regression by
   Local Fitting: Methods, Prospectives and Computational
   Algorithms. \emph{Journal of Econometrics}, \bold{37}, pp. 87--114.

   M. Birattari, G. Bontempi, and H. Bersini (1999) Lazy learning meets
   the recursive least squares algorithm. \emph{Advances in Neural
   Information Processing Systems 11}, pp. 375--381. MIT Press.

   G. Bontempi, M. Birattari, and H. Bersini (1999) Lazy learning for
   modeling and control design. \emph{International Journal of Control},
   \bold{72}(7/8), pp. 643--658.

   G. Bontempi, M. Birattari, and H. Bersini (1999) Local learning for
   iterated time-series prediction. \emph{International Conference on
     Machine Learning}, pp. 32--38. Morgan Kaufmann.
 
}

\author{Mauro Birattari and Gianluca Bontempi}

\seealso{\code{\link{lazy.control}}, \code{\link{predict.lazy}}}

\examples{
data(cars)
cars.lazy <- lazy(dist ~ speed, cars)
predict(cars.lazy, data.frame(speed = seq(5, 30, 1)))
}

\keyword{regression}




\eof
% ------------------------------------------------------------------------- %
% lazy.control.Rd                           Set parameters fo lazy learning %
% ------------------------------------------------------------------------- %
                                                                             
% ========================================================================= %
% Lazy learning for local regression                                        %
% ------------------------------------------------------------------------- %
% Copyright (C) 1999, 2003 Mauro Birattari and Gianluca Bontempi            %
% ========================================================================= %
% This program is free software; you can redistribute it and/or modify it   %
% under the terms of the GNU General Public License as published by the     %
% Free Software Foundation; either version 2 of the License, or (at your    %
% option) any later version.                                                %
%                                                                           %
% This program is distributed in the hope that it will be useful, but       %
% WITHOUT ANY WARRANTY; without even the implied warranty of                %
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         %
% General Public License for more details.                                  %
%                                                                           %
% You should have received a copy of the GNU General Public License along   %
% with this program; if not, write to the Free Software Foundation, Inc.,   %
% 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.                  %
% ========================================================================= %

% ========================================================================= %
%             Mauro Birattari                     Gianluca Bontempi         %
%                IRIDIA                     Departement d'Informatique      %
%    Universite' Libre de Bruxelles       Universite' Libre de Bruxelles    %
%             mbiro@ulb.ac.be                     gbonte@ulb.ac.be          %
% ========================================================================= %

% $Id: lazy.control.Rd,v 1.1 2003/10/02 16:19:31 mbiro Exp $ %

\name{lazy.control}
\alias{lazy.control}

\title{Set parameters for lazy learning}

\description{Set control parameters for a lazy learning object.}

\usage{
lazy.control(conIdPar=NULL, linIdPar=1, quaIdPar=NULL,
                distance=c("manhattan","euclidean"), metric=NULL,
                   cmbPar=1, lambda=1e+06)
}


\arguments{

  \item{conIdPar}{Parameter controlling the number of neighbors to be used
    for identifying and validating constant models. \code{conIdPar} can assume
    different forms:
    \describe{
      \item{\code{conIdPar=c(idm0,idM0,valM0)}:}{ In this case,
	\code{idm0:idM0} is the range in which the best number of
	neighbors is searched when identifying the local polynomial
	models of degree 0 and where \code{valM0} is the maximum
	number of neighbors used for their validation.  This means
	that the constant models identified with \code{k} neighbors,
	are validated on the first \code{v} neighbors, where
	\code{v=min(k,valM0)}.  If \code{valM0=0}, \code{valM0} is set
	to \code{idMO}: see next case for details.}

      \item{\code{conIdPar=c(idm0,idM0)}:}{ Here \code{idm0} and
	\code{idM0} have the same role as in previous case, and
	\code{valM0} is by default set to \code{idM0}: each model is
	validated on all the neighbors used in identification.}

      \item{\code{conIdPar=p}:}{ Here \code{idmO} and \code{idMO} are
        obtained according to the following formulas: \code{idm0=3}
        and \code{idMX=5*p}. Recommended choice: \code{p=1}. As far as
        the quantity \code{valM0} is concerned, it gets the default
        value as in previous case.}

      \item{\code{conIdPar=NULL}:}{ No constant model is considered.}}}

  \item{linIdPar}{Parameter controlling the number of neighbors to be used
    for identifying and validating linear models. \code{linIdPar} can assume
    different forms:
    \describe{
      \item{\code{linIdPar=c(idm1,idM1,valM1)}:}{ In this case,
	\code{idm1:idM1} is the range in which the best number of
	neighbors is searched when identifying the local polynomial
	models of degree 1 and where \code{valM1} is the maximum
	number of neighbors used for their validation.  This means
	that the linear models identified with \code{k} neighbors, are
	validated on the first \code{v} neighbors, where
	\code{v=min(k,valM1)}.  If \code{valM1=0}, \code{valM1} is set
	to \code{idM1}: see next case for details.}

      \item{\code{linIdPar=c(idm1,idM1)}:}{ Here
	\code{idm1} and \code{idM1} have the same role as in previous
	case, and \code{valM1} is by default set to \code{idM1}: each
	model is validated on all the neighbors used in identification.}

      \item{\code{linIdPar=p}:}{ Here \code{idmO} and \code{idMO} are
        obtained according to the following formulas:
        \code{idm1=3*noPar} and \code{idM1=5*p*noPar}, where
        \code{noPar=nx+1} is the number of parameter of the polynomial
        model of degree 1, and \code{nx} is the dimensionality of the
        input space. Recommended choice: \code{p=1}. As far as the
        quantity \code{valM1} is concerned, it gets the default value
        as in previous case.}

      \item{\code{linIdPar=NULL}:}{ No linear model is considered.}}}

  \item{quaIdPar}{Parameter controlling the number of neighbors to be
    used for identifying and validating quadratic
    models. \code{quaIdPar} can assume different forms:
    \describe{
      \item{\code{quaIdPar=c(idm2,idM2,valM2)}:}{ In this case,
	\code{idm2:idM2} is the range in which the best number of
	neighbors is searched when identifying the local polynomial
	models of degree 2 and where \code{valM2} is the maximum
	number of neighbors used for their validation.  This means
	that the quadratic models identified with \code{k} neighbors, are
	validated on the first \code{v} neighbors, where
	\code{v=min(k,valM2)}.  If \code{valM2=0}, \code{valM2} is set
	to \code{idM2}: see next case for details.}

      \item{\code{quaIdPar=c(idm2,idM2)}:}{ Here
	\code{idm2} and \code{idM2} have the same role as in previous
	case, and \code{valM2} is by default set to \code{idM2}: each
	model is validated on all the neighbors used in identification.}

      \item{\code{quaIdPar=p}:}{ Here \code{idmO} and \code{idMO} are
        obtained according to the following formulas:
        \code{idm2=3*noPar} and \code{idM2=5*p*noPar}, where in this
	case the number of parameters is 
        \code{noPar=(nx+1)*(nx+2)/2}, and \code{nx} is the
        dimensionality of the input space. Recommended choice:
        \code{p=1}. As far as the quantity \code{valM2} is concerned,
        it gets the default value as in previous case.}

      \item{\code{quaIdPar=NULL}:}{ No quadratic model is considered.}}}

  \item{distance}{The distance metric: can be \code{manhattan} or
    \code{euclidean}.}

  \item{metric}{Vector of \code{n} elements. Weights used to evaluate
    the distance between query point and neighbors.}
  
  \item{cmbPar}{Parameter controlling the local combination of
    models. \code{cmbPar} can assume different forms:
    \describe{
      \item{\code{cmbPar=c(cmb0,cmb1,cmb2)}:}{ In this case, \code{cmbX}
         is the number of polynomial models of degree \code{X} that will
         be included in the local combination. Each local model will be
         therfore a combination of \emph{the best} \code{cmb0}
         \emph{models of degree 0, the best} \code{cmb1} \emph{models of
         degree 1, and the best} \code{cmb2} \emph{models of degree 2}
         identified as specified by \code{idPar}.}

      \item{\code{cmbPar=cmb}:}{ Here \code{cmb} is the number of models
         that will be combined, disregarding any constraint on the
         degree of the models that will be considered.  Each local model
         will be therfore a combination of \emph{the best} \code{cmb}
         \emph{models}, identified as specified by \code{id_par}.}}}

  \item{lambda}{Initialization of the diagonal elements of the local
    variance/covariance matrix for Ridge Regression.}
}


\value{The output of \code{lazy.control} is a list containing the
  following components: \code{conIdPar}, \code{linIdPar}, \code{quaIdPar},
  \code{distance}, \code{metric}, \code{cmbPar}, \code{lambda}.
}

\author{Mauro Birattari and Gianluca Bontempi}

\seealso{\code{\link{lazy}}, \code{\link{predict.lazy}}}

\keyword{regression}




\eof
% ------------------------------------------------------------------------- %
% predict.lazy.Rd                         Predict method for lazy learning  %
% ------------------------------------------------------------------------- %
                                                                             
% ========================================================================= %
% Lazy learning for local regression                                        %
% ------------------------------------------------------------------------- %
% Copyright (C) 1999, 2003 Mauro Birattari and Gianluca Bontempi            %
% ========================================================================= %
% This program is free software; you can redistribute it and/or modify it   %
% under the terms of the GNU General Public License as published by the     %
% Free Software Foundation; either version 2 of the License, or (at your    %
% option) any later version.                                                %
%                                                                           %
% This program is distributed in the hope that it will be useful, but       %
% WITHOUT ANY WARRANTY; without even the implied warranty of                %
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         %
% General Public License for more details.                                  %
%                                                                           %
% You should have received a copy of the GNU General Public License along   %
% with this program; if not, write to the Free Software Foundation, Inc.,   %
% 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.                  %
% ========================================================================= %

% ========================================================================= %
%             Mauro Birattari                     Gianluca Bontempi         %
%                IRIDIA                     Departement d'Informatique      %
%    Universite' Libre de Bruxelles       Universite' Libre de Bruxelles    %
%             mbiro@ulb.ac.be                     gbonte@ulb.ac.be          %
% ========================================================================= %

% $Id: predict.lazy.Rd,v 1.3 2003/12/09 15:32:27 mbiro Exp $ %

\name{predict.lazy}
\alias{predict.lazy}

\title{Predict method for lazy learning}

\description{Obtains predictions from a lazy learning object}

\usage{%
\method{predict}{lazy}(object, newdata=NULL,
          t.out=FALSE, k.out=FALSE,
            S.out=FALSE, T.out=FALSE, I.out=FALSE, \dots)
}

\arguments{
  \item{object}{Object of class inheriting from \code{lazy}.}

  \item{newdata}{Data frame (or matrix, vector, etc\ldots) defining of the
    query points for which a prediction is to be produced.}
  
  \item{t.out}{Logical switch indicating if the function should return
    the parameters of the local models used to perform each estimation.}

  \item{k.out}{Logical switch indicating if the function should return
    the number of neighbors used to perform each estimation.}

  \item{S.out}{Logical switch indicating if the function should return
    the estimated variance of the prediction suggested by \bold{all} the
    models identified for each query point.}

  \item{T.out}{Logical switch indicating if the function should return
    the parameters of \bold{all} the models identified for each query point.}
  
  \item{I.out}{Logical switch indicating if the function should return
    the index \code{i} of all the samples \code{(X[i,],Y[i])} used to
    perform each estimation.}

  \item{\dots}{Arguments passed to or from other methods.}
}

\value{The output of the method is a list containing the following
  components:
  
  \item{h}{Vector of \code{q} elements, where \code{q} is the number of
    rows in \code{newdata}, i.e. the number of query points. The element
    in position \code{i} is the estimate of the value of the unknown function
    in the query point \code{newdata[i,]}.  The component \code{h} is
    always returned.}
  
  \item{t}{Matrix of \code{z*q} elements, where \code{z=z2} i.e., number of
    parameters of a quadratic model if at least one model of degree 2
    was identified (see \code{quaIdPar} in \code{\link{lazy.control}}),
    otherwise \code{z=z1} i.e.,
    number of parameters of a linear model if at least one model of
    degree 1 was identified (see \code{linIdPar} in
    \code{\link{lazy.control}}), or \code{z=1} if only
    models of degree 0 where considered. In the general case,
    the elements of the vector \code{t[,j]=c(a0, a1,\ldots, an, a11,
      a12,\ldots, a22, a23,\ldots, a33, a34,\ldots, ann)} are
    the parameters of the local  model used for estimating
    the function in the \code{j}th query point: the cross-terms terms
    \code{a11,a12,\ldots,ann} wil be missing if no quadratic model is
    identified and the terms \code{a1,\ldots,an}, will be missing if
    no linear model is identified. If, according to \code{cmbPar} (see
    \code{\link{lazy.control}}), estimations are to be performed by a
    combination of models, the elements of \code{t[,j]} are a weighted
    average of the parameters
    of the selected models where the weight of each model is the
    inverse of the a leave-one-out estimate of the variances of the
    model itself. \bold{REMARK:} a translation of the axes is considered 
    which centers all the local models in the respective query point.}
  
  \item{k}{Vector of \code{q} elements. Selected number of neighbors
    for each query point. If, according to \code{cmbPar} (see
    \code{\link{lazy.control}}), a local
    combination of models is considered, \code{k[j]} is the largest
    value among the number of neighbors used by the selected models
    for estimating the value in the \code{j}th query point.}
  
  \item{S}{List of up to 3 components: Each component is a matrix
    containing an estimate, obtained through a leave-one-out
    cross-valication, of the variance of local models.
    \describe{
      \item{\code{con}}{Matrix of \code{idM0*q} elements, where
        \code{idM0} is the maximum number of neighbors used to fit local
        polynomial models of degree 0 (see \code{\link{lazy.control}}):
        Estimated
        variance of \bold{all} the constant
        models identified for each query point. If no constant model
	is identified (see \code{conIdPar} and \code{cmbPar} in
        \code{\link{lazy.control}}) \code{S$con} is not returned.}
      \item{\code{lin}}{Matrix of \code{idM1*q} elements, where
        \code{idM1} is the maximum number of neighbors used to fit local
        polynomial models of degree 1 (see \code{\link{lazy.control}}):
        Estimated
        variance of \bold{all} the linear
        models identified for each query point. If no linear model
	is identified (see \code{linIdPar} and \code{cmbPar} in
        \code{\link{lazy.control}}) \code{S$lin} is not returned.}
      \item{\code{qua}}{Matrix of \code{idM2*q} elements, where
        \code{idM1} is the maximum number of neighbors used to fit local
        polynomial models of degree 1 (see \code{\link{lazy.control}}):
        Estimated variance of \bold{all} the quadratic
        models identified for each query point. If no quadratic model
	is identified (see \code{quaIdPar} and \code{cmbPar} in
        \code{\link{lazy.control}}) \code{S$qua} is not returned.}}
    The component \code{S} is returned only if \code{S.out=TRUE} in
    the function call.}
  
  \item{T}{List of up to 3 components:
    \describe{
      \item{\code{con}}{Array of \code{z0*idM0*q} elements, where
	\code{z0=1} is the number of parameters of a model of degree
	0. The element \code{T$con[1,i,j]=a0} is the single parameter of
	the local model identified on \code{i} neighbors of the
	\code{q}th query point.}
      \item{\code{lin}}{Array of \code{z1*idM1*q} elements where,  if
	\code{n} is the dimensionality of the input space,
	\code{z1=n+1} is the number of parameter of a model of degree
	1. The vector \code{T$lin[,i,j]=c(a0,a1,\ldots,an)} is the
	vector of parameters of
	the local model identified on \code{i} neighbors of the
	\code{q}th query point. In particular, \code{a0} is the
	constant term, \code{a1} is the parameter associated with the
	first input variable and so on.}
      \item{\code{qua}}{Array of \code{z2*idM2*q} elements where, if
	\code{n} is the dimensionality of the input space,
	\code{z2=(n+1)*(n+2)/2} is the number of parameter of a model
	of degree 2. The vector
	\code{T$qua[,i,j]=c(a0, a1,\ldots, an, a11, 
	  a12,\ldots, a22, a23,\ldots, a33, a34,\ldots, ann)}
	is the vector of parameters of the local quadratic model
	identified on \code{i} neighbors of the \code{q}th query
	point. In particular, \code{a0,\ldots,a1} are the constant and
	liner parameters as in \code{T$lin}, while
	\code{a11,a12,\ldots,ann} are the quadratic ones: \code{a11}
	is associated with the quadratic term \code{x1^2}, \code{a12}
	with the cross-term \code{x1*x2}, and so on.}}
    \bold{REMARK:} a translation of the axes is considered 
    which centers all the local models in the respective query
    point. The component \code{T} is returned only if
    \code{T.out=TRUE} in the function call.}
  
  \item{I}{Matrix of \code{idM*q} elements, where \code{idM} is the
    largest of \code{idM0}, \code{idM1}, and \code{idM2}. Contains the
    index of the neighbors of each query point in \code{newdata}.
    In particular, \code{I[i,j]} is the \code{i}th nearest neighbor of
    the \code{q}th query point.}
}

\author{Mauro Birattari and Gianluca Bontempi}

\seealso{\code{\link{lazy}}, \code{\link{lazy.control}}}

\examples{
data(cars)
cars.lazy <- lazy(dist ~ speed, cars)
predict(cars.lazy, data.frame(speed = seq(5, 30, 1)))
}


\keyword{regression}

\eof
